@@ -12,15 +12,19 @@ import zerorpc
 import zmq
 import logging
 logging.basicConfig()
-import StringIO
 import json
 import hashlib
 import inspect
 import funcsigs
 import ConfigParser
 import magic
-from colorama import Fore, Style
+from colorama import Fore as F, Style
 import datetime
+import lz4
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 # Workbench server imports
 try:
@@ -109,9 +113,6 @@ class WorkBench(object):
                 the md5 of the sample.
         """
 
-        # Store the sample
-        # md5 = self.data_store.store_sample(input_bytes, filename, type_tag)
-
         # If the sample comes in with an unknown type_tag try to determine it
         if type_tag == 'unknown':
             print 'Info: Unknown File -- Trying to Determine Type...'
@@ -124,10 +125,18 @@ class WorkBench(object):
                 else:
                     print 'Alert: Failed to Determine Type for %s' % filename
                     exit(1) # Temp
-            
-        # Temp
+
+        # Do we have a compressed sample? If so decompress it
+        if type_tag == 'lz4':
+            input_bytes = lz4.loads(input_bytes)
+
+        # Store the sample
         md5 = self.data_store.store_sample(input_bytes, filename, type_tag)
 
+        # Add the type_tags to tags
+        if type_tag != 'lz4':
+            self.add_tags(md5, type_tag)
+
         return md5
 
     def get_sample(self, md5):
@@ -143,18 +152,32 @@ class WorkBench(object):
         # First we try a sample, if we can't find one we try getting a sample_set.
         sample = self.data_store.get_sample(md5)
         if not sample:
-            return self.get_sample_set(md5)
+            return {'sample_set': {'md5_list': self.get_sample_set(md5)}}
         return {'sample': sample}
 
+    def is_sample_set(self, md5):
+        """ Does the md5 represent a sample_set?
+            Args:
+                md5: the md5 of the sample_set
+            Returns:
+                True/False
+        """
+        try:
+            self.get_sample_set(md5)
+            return True
+        except WorkBench.DataNotFound:
+            return False
+
     def get_sample_window(self, type_tag, size):
         """ Get a sample from the DataStore.
             Args:
                 type_tag: the type of samples ('pcap','exe','pdf')
                 size: the size of the window in MegaBytes (10 = 10MB)
             Returns:
-                A list of md5s representing the newest samples within the size window
+                A sample_set handle which represents the newest samples within the size window
         """
-        return self.data_store.get_sample_window(type_tag, size)
+        md5_list = self.data_store.get_sample_window(type_tag, size)
+        return self.store_sample_set(md5_list)
 
     def has_sample(self, md5):
         """ Do we have this sample in the DataStore.
@@ -165,17 +188,6 @@ class WorkBench(object):
         """
         return self.data_store.has_sample(md5)
 
-    def list_samples(self, predicate=None):
-        """List all samples that meet the predicate or all if predicate is not specified.
-
-            Args:
-                predicate: Match samples against this predicate (or all if not specified)
-
-            Returns:
-                List of dictionaries with matching samples {'md5':md5, 'filename': 'foo.exe', 'type_tag': 'exe'}
-        """
-        return self.data_store.list_samples(predicate)
-
     def combine_samples(self, md5_list, filename, type_tag):
         """Combine samples together. This may have various use cases the most significant 
            involving a bunch of sample 'chunks' got uploaded and now we combine them together
@@ -221,7 +233,7 @@ class WorkBench(object):
         type_tag = sample['type_tag']
         if type_tag == 'bro':
             bro_log = bro_log_reader.BroLogReader(convert_datetimes=False)
-            mem_file = StringIO.StringIO(raw_bytes)
+            mem_file = StringIO(raw_bytes)
             generator = bro_log.read_log(mem_file)
             return generator
         elif type_tag == 'els_query':
@@ -241,6 +253,28 @@ class WorkBench(object):
         else:
             raise RuntimeError('Cannot stream file %s with type_tag:%s' % (md5, type_tag))
 
+    def get_dataframe(self, md5, compress='lz4'):
+        """Return a dataframe from the DataStore. This is just a convenience method
+           that uses get_sample internally. 
+            Args:
+                md5: the md5 of the dataframe
+                compress: compression to use: (defaults to 'lz4' but can be set to None)
+            Returns:
+                A msgpack'd Pandas DataFrame
+            Raises:
+                Workbench.DataNotFound if the dataframe is not found.
+        """
+        # First we try a sample, if we can't find one we try getting a sample_set.
+        sample = self.data_store.get_sample(md5)
+        if not sample:
+            raise WorkBench.DataNotFound("Could not find %s in the data store", md5)
+        if not compress:
+            return sample['raw_bytes']
+        else:
+            compress_df = lz4.dumps(sample['raw_bytes'])
+            print 'Info: DataFrame compression %.0f%%' % (len(compress_df)*100.0/float(len(sample['raw_bytes'])))
+            return compress_df
+
     def guess_type_tag(self, input_bytes):
         """ Try to guess the type_tag for this sample """
         mime_to_type = {'application/jar': 'jar',
@@ -271,6 +305,31 @@ class WorkBench(object):
                 print 'Alert: Sample Type could not be Determined'
                 return 'unknown'
 
+    def add_tags(self, md5, tags):
+        """Add tags to this sample"""
+        if not tags: return
+        tag_set = set(self.get_tags(md5)) if self.get_tags(md5) else set()
+        if isinstance(tags, str):
+            tags = [tags]
+        for tag in tags:
+            tag_set.add(tag)
+        self.data_store.store_work_results({'tags': list(tag_set)}, 'tags', md5)
+
+    def set_tags(self, md5, tags):
+        """Set the tags for this sample"""
+        if isinstance(tags, str):
+            tags = [tags]
+        tag_set = set(tags)
+        self.data_store.store_work_results({'tags': list(tag_set)}, 'tags', md5)
+
+    def get_tags(self, md5):
+        """Get tags for this sample"""
+        tag_data = self.data_store.get_work_results('tags', md5)
+        return tag_data['tags'] if tag_data else None
+
+    def get_all_tags(self):
+        """Get tags for this sample"""
+        return self.data_store.tags_all()
 
 
     #######################
@@ -308,7 +367,7 @@ class WorkBench(object):
         # Okay now index the data
         self.indexer.index_data(data, index_name=index_name, doc_type='unknown')
 
-    def search(self, index_name, query):
+    def search_index(self, index_name, query):
         """ Search a particular index in the Indexer
             Args:
                 index_name: the name of the index
@@ -376,7 +435,22 @@ class WorkBench(object):
         self.plugin_manager.load_all_plugins()
 
         # Store information about commands and workbench
-        self._store_information()        
+        self._store_information()
+
+    def clear_worker_output(self):
+        """Drops all of the worker output collections
+            Args:
+                None
+            Returns:
+                Nothing
+        """
+        self.data_store.clear_worker_output()
+
+        # Have the plugin manager reload all the plugins
+        self.plugin_manager.load_all_plugins()
+
+        # Store information about commands and workbench
+        self._store_information()
 
 
     #######################
@@ -386,70 +460,68 @@ class WorkBench(object):
         """ Make a work request for an existing stored sample.
             Args:
                 worker_name: 'strings', 'pe_features', whatever
-                md5: the md5 of the sample
-                subkeys: just return a subfield e.g. 'foo' or 'foo.bar' (None for all) 
+                md5: the md5 of the sample (or sample_set!)
+                subkeys: just get a subkey of the output: 'foo' or 'foo.bar' (None for all) 
             Returns:
-                The output of the worker or just the subfield of the worker output
+                The output of the worker.
         """
 
-        # Check valid
-        if worker_name not in self.plugin_meta.keys():
-            raise RuntimeError('Invalid work request for class %s (not found)' % (worker_name))
-
-        # Get results (even if we have to wait for them)
-        # Note: Yes, we're going to wait. Gevent concurrent execution will mean this
-        #       code gets spawned off and new requests can be handled without issue.
+        # Pull the worker output
         work_results = self._recursive_work_resolver(worker_name, md5)
 
-        # Subkeys? (Fixme this is super klutzy)
+        # Subkeys (Fixme this is super klutzy)
         if subkeys:
+            if isinstance(subkeys, str):
+                subkeys = [subkeys]
             try:
                 sub_results = {}
                 for subkey in subkeys:
                     tmp = work_results[worker_name]
-                    for key in subkey.split('.'):
+
+                    # Traverse any subkeys
+                    for key in subkey.split('.')[:-1]:
                         tmp = tmp[key]
-                        sub_results[key] = tmp
+
+                    # Last subkey
+                    key = subkey.split('.')[-1]
+                    if key == '*':
+                        for key in tmp.keys():
+                            sub_results[key] = tmp[key]
+                    else:
+                        sub_results[key] = tmp[key]
+
+                # Set the output
                 work_results = sub_results
+
             except (KeyError, TypeError):
                 raise RuntimeError('Could not get one or more subkeys for: %s' % (work_results))
 
         # Clean it and ship it
-        work_results = self.data_store.clean_for_serialization(work_results)
-        return work_results
+        return self.data_store.clean_for_serialization(work_results)
 
     @zerorpc.stream
-    def batch_work_request(self, worker_name, kwargs=None):
-        """Make a batch work request for an existing set of stored samples.
-
-        A subset of sample can be specified with kwargs.
-
-        Args:
-            worker_name: 'strings', 'pe_features', whatever
-            kwargs: a way of specifying subsets of samples ({} for all)
-                type_tag: subset based on sample type (e.g. type_tag='exe')
-                md5_list: subset just the samples in this list
-                subkeys: return just this subkey (e.g. 'foo' or 'foo.bar')
-        Returns:
-            A generator that yields rows of worker output or subfields of the worker output
+    def set_work_request(self, worker_name, sample_set, subkeys=None):
+        """ Make a work request for an existing stored sample (or sample_set).
+            Args:
+                worker_name: 'strings', 'pe_features', whatever
+                sample_set: the md5 of a sample_set in the Workbench data store
+                subkeys: just get a subkey of the output: 'foo' or 'foo.bar' (None for all) 
+            Returns:
+                The output is a generator of the results of the worker output for the sample_set
         """
-        type_tag = kwargs.get('type_tag',None) if kwargs else None
-        md5_list = kwargs.get('md5_list',None) if kwargs else None
-        subkeys = kwargs.get('subkeys',None) if kwargs else None
-
-        # If no md5_list specified put all samples (of type type_tag if not None)
-        if not md5_list:
-            md5_list = self.data_store.all_sample_md5s(type_tag) 
 
+        # Does worker support sample_set_input?
+        if self.plugin_meta[worker_name]['sample_set_input']:
+            yield self.work_request(worker_name, sample_set, subkeys)
+ 
         # Loop through all the md5s and return a generator with yield
-        for md5 in md5_list:
-            try:
+        else:
+            md5_list = self.get_sample_set(sample_set)
+            for md5 in md5_list:
                 if subkeys:
                     yield self.work_request(worker_name, md5, subkeys)
                 else:
                     yield self.work_request(worker_name, md5)[worker_name]
-            except KeyError:
-                continue
 
     def store_sample_set(self, md5_list):
         """ Store a sample set (which is just a list of md5s).
@@ -460,26 +532,48 @@ class WorkBench(object):
                 md5_list: a list of the md5s in this set (all must exist in data store)
 
             Returns:
-                The md5 of the set (the actual md5 of the set
+                The md5 of the set (the actual md5 of the set)
         """
+
+        # Sanity check
+        if not md5_list:
+            print 'Warning: Trying to store an empty sample_set'
+            return None
+
+        # Remove any duplicates
+        md5_list = list(set(md5_list))
+
         for md5 in md5_list:
             if not self.has_sample(md5):
-                raise RuntimeError('Sample not found all items in sample_set\
-                                    must be in the datastore: %s (not found)' % (md5))
+                raise RuntimeError('%s: Not found! All items in sample_set must be in the datastore' % (md5))
         set_md5 = hashlib.md5(str(md5_list)).hexdigest()
         self._store_work_results({'md5_list':md5_list}, 'sample_set', set_md5)
         return set_md5
 
-    def get_sample_set(self, md5):
-        """ Store a sample set (which is just a list of md5s).
+    def generate_sample_set(self, tags=None):
+        """Generate a sample_set that maches the tags or all if tags are not specified.
 
             Args:
-                md5_list: a list of the md5s in this set (all must exist in data store)
+                tags: Match samples against this tag list (or all if not specified)
 
             Returns:
-                The md5 of the set (the actual md5 of the set
+                The sample_set of those samples matching the tags
         """
-        return self._get_work_results('sample_set', md5)
+        if isinstance(tags, str):
+            tags = [tags]
+        md5_list = self.data_store.tag_match(tags)
+        return self.store_sample_set(md5_list)
+
+    def get_sample_set(self, md5):
+        """ Retrieve a sample set (which is just a list of md5s).
+
+            Args:
+                md5: the md5 of the sample_set (returned with the 'store_sample_set' call)
+
+            Returns:
+                The list of md5s that comprise the sample_set
+        """
+        return self.data_store.clean_for_serialization(self._get_work_results('sample_set', md5))['sample_set']['md5_list']
 
     @zerorpc.stream
     def stream_sample_set(self, md5):
@@ -495,7 +589,7 @@ class WorkBench(object):
             yield md5
 
     def get_datastore_uri(self):
-        """ Gives you the current datastore URL.
+        """ Gives you the current datastore URI.
 
             Args:
                 None
@@ -518,52 +612,52 @@ class WorkBench(object):
         # so we'll catch the exception and push back an object that
         # indicates we didn't find what they were asking for
         try:
-            return self.work_request('help_cli', topic)['help_cli']['help']
+            return self.work_request('help_formatter', topic)['help_formatter']['help']
         except WorkBench.DataNotFound as e:
 
             # Okay this is a bit tricky we want to give the user a nice error
             # message that has both the md5 of what they were looking for and
             # a nice informative message that explains what might have happened
-            sample_md5 = e.args[0]['md5']
-            return '%s%s not found:\n\t%s' % (Fore.YELLOW, sample_md5, e.message())
+            sample_md5 = e.args[0]
+            return '%s%s\n\t%s%s%s' % (F.YELLOW, sample_md5, F.GREEN, e.message(), F.RESET)
 
     # Fixme: These are internal methods that basically just provide help text
     def _help_workbench(self):
         """ Help on Workbench """
-        help = '%sWelcome to Workbench Help:%s' % (Fore.YELLOW, Fore.RESET)
-        help += '\n\t%s- workbench.help(topic) %s where topic can be a help, command or worker' % (Fore.GREEN, Fore.BLUE)
-        help += '\n\t%s- workbench.help(\'basic\') %s for getting started help' % (Fore.GREEN, Fore.BLUE)
-        help += '\n\t%s- workbench.help(\'commands\') %s for help on workbench commands' % (Fore.GREEN, Fore.BLUE)
-        help += '\n\t%s- workbench.help(\'workers\') %s for help on available workers' % (Fore.GREEN, Fore.BLUE)
-        help += '\n\n%sSee http://github.com/SuperCowPowers/workbench for more information\n%s' % (Fore.YELLOW, Fore.RESET)
+        help = '%sWelcome to Workbench Help:%s' % (F.YELLOW, F.RESET)
+        help += '\n\t%s- workbench.help(\'basic\') %s for getting started help' % (F.GREEN, F.BLUE)
+        help += '\n\t%s- workbench.help(\'workers\') %s for help on available workers' % (F.GREEN, F.BLUE)
+        help += '\n\t%s- workbench.help(\'commands\') %s for help on workbench commands' % (F.GREEN, F.BLUE)
+        help += '\n\t%s- workbench.help(topic) %s where topic can be a help, command or worker' % (F.GREEN, F.BLUE)
+        help += '\n\n%sSee http://github.com/SuperCowPowers/workbench for more information\n%s' % (F.YELLOW, F.RESET)
         return help
 
     def _help_basic(self):
         """ Help for Workbench Basics """
-        help =  '%sWorkbench: Getting started...' % (Fore.YELLOW)
-        help += '\n%sLoad in a sample:'  % (Fore.GREEN)
-        help += '\n\t%s$ load_sample /path/to/file' % (Fore.BLUE)
-        help += '\n\n%sNotice the prompt now shows the md5 of the sample...'% (Fore.YELLOW)
-        help += '\n%sRun workers on the sample:'  % (Fore.GREEN)
-        help += '\n\t%s$ meta or view or whatever... %s' % (Fore.BLUE, Fore.RESET)
+        help =  '%sWorkbench: Getting started...' % (F.YELLOW)
+        help += '\n%sStore a sample into Workbench:'  % (F.GREEN)
+        help += '\n\t%s$ workbench.store_sample(raw_bytes, filename, type_tag)' % (F.BLUE)
+        help += '\n\n%sNotice store_sample returns an md5 of the sample...'% (F.YELLOW)
+        help += '\n%sRun workers on the sample (view, meta, whatever...):'  % (F.GREEN)
+        help += '\n\t%s$ workbench.work_request(\'view\', md5)%s' % (F.BLUE, F.RESET)
         return help
 
     def _help_commands(self):
         """ Help on all the available commands """
         help =  'Workbench Commands:'
         for command in self.list_all_commands():
-            full_help = self.work_request('help_cli', command)['help_cli']['help']
-            compact_help = full_help.split('\n')[:3]
-            help += '\n\t%s' % '\n'.join(compact_help)
+            full_help = self.work_request('help_formatter', command)['help_formatter']['help']
+            compact_help = full_help.split('\n')[:2]
+            help += '\n\n%s' % '\n'.join(compact_help)
         return help
 
     def _help_workers(self):
         """ Help on all the available workers """
         help =  'Workbench Workers:'
         for worker in self.list_all_workers():
-            full_help = self.work_request('help_cli', worker)['help_cli']['help']
+            full_help = self.work_request('help_formatter', worker)['help_formatter']['help']
             compact_help = full_help.split('\n')[:4]
-            help += '\n\t%s' % '\n'.join(compact_help)
+            help += '\n\n%s' % '\n'.join(compact_help)
         return help
 
 
@@ -670,8 +764,8 @@ class WorkBench(object):
                 The newest modification time of any worker in the work chain. 
         """
 
-        # Bottom out on sample or info
-        if worker_name=='sample' or worker_name=='info':
+        # Bottom out on sample, info or tags
+        if worker_name=='sample' or worker_name=='info' or worker_name=='tags':
             return datetime.datetime(1970, 1, 1)
 
         my_mod_time = self._get_work_results('info', worker_name)['info']['mod_time']
@@ -696,6 +790,10 @@ class WorkBench(object):
         if worker_name == 'info':
             return self._get_work_results('info', md5)
 
+        # Looking for tags?
+        if worker_name == 'tags':
+            return self._get_work_results('tags', md5)
+
         # Do I actually have this plugin? (might have failed, etc)
         if (worker_name not in self.plugin_meta):
             print 'Alert: Request for non-existing or failed plugin: %s' % (worker_name)
@@ -708,8 +806,8 @@ class WorkBench(object):
             work_chain_mod_time = self._work_chain_mod_time(worker_name)
             if work_chain_mod_time < work_results[collection]['__time_stamp']:
                 return work_results
-            else:
-                print 'Notice: %s work_chain is newer than data' % (worker_name)
+            elif self.VERBOSE:
+                print 'VERBOSE: %s work_chain is newer than data' % (worker_name)
         except WorkBench.DataNotFound:
             if self.VERBOSE:
                 print 'Verbose: %s data not found generating' % (worker_name)
