@@ -23,7 +23,7 @@ def test():
     # Generate input for the worker
     import os
     data_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../data/log/system.log')
-    md5 = workbench.store_sample('system.log', open(data_path, 'rb').read(), 'log')
+    md5 = workbench.store_sample(open(data_path, 'rb').read(), 'system.log', 'log')
     input_data = workbench.work_request('log_meta', md5)
 
     # Execute the worker (unit test)
